# MQFed+FHE: Multimodal Quantum Federated Learning with Secure Fully Homomorphic Encryption (FHE)

## Installation

### Install Dependencies

#### For CPU

```bash
conda create -n fed python=3.10.12 anaconda
conda init
conda activate fed
bash run-cpu.sh
```

#### For GPU

```bash
conda create -n fed python=3.10.12 anaconda
conda init
conda activate fed
bash run-gpu.sh
```

### Running Experiments

Choose the appropriate notebook based on your dataset and encryption preference:

#### FHE-enabled Quantum Federated Learning

- **CIFAR-10 Dataset:**
  - Notebook: `src/FHE_FedQNN_CIFAR.ipynb`
  - Description: CIFAR-10 consists of 60,000 32x32 color images in 10 classes, with 6,000 images per class. It is widely used for image classification tasks.

- **DNA Sequence Dataset:**
  - Notebook: `src/FHE_FedQNN_DNA.ipynb`
  - Description: This dataset includes DNA sequences used for various biological and genetic studies, focusing on sequence classification and pattern recognition.

- **MRI Scan Dataset:**
  - Notebook: `src/FHE_FedQNN_MRI.ipynb`
  - Description: This dataset contains MRI scans used for medical image analysis, particularly for detecting and diagnosing conditions based on scan data.

- **PCOS Dataset:**
  - **Notebook:** `src/FHE_FedQNN_PCOS.ipynb`
  - **Description:** This dataset is used for analyzing Polycystic Ovary Syndrome (PCOS). It is employed for developing and assessing models aimed at detecting and diagnosing PCOS based on this data.

- **DNA+MRI Multimodal Dataset:**
  - **Notebook:** `src/FHE_FedQNN_DNA+MRI.ipynb`
  - **Description:** It is used as a MoE with Multimodaility leveraging both DNA Sequence and MRI scans data to develop and evaluate models for detecting and interpreting tumors and dna classes.  

#### Standard Quantum Federated Learning

- **CIFAR-10 Dataset:**
  - Notebook: `src/Standard_FedQNN_CIFAR.ipynb`
  - Description: The same CIFAR-10 dataset, utilized without the FHE layer, for benchmarking and comparison.

- **DNA Sequence Dataset:**
  - Notebook: `src/Standard_FedQNN_DNA.ipynb`
  - Description: The same DNA sequence dataset, used without FHE for standard federated learning experiments.

- **MRI Scan Dataset:**
  - Notebook: `src/Standard_FedQNN_MRI.ipynb`
  - Description: The same MRI scan dataset, used without FHE to evaluate the performance of standard federated learning models.

- **PCOS Dataset:**
  - **Notebook:** `src/Standard_FedQNN_PCOS.ipynb`
  - **Description:** The same PCOS dataset, used without FHE to evaluate the performance of standard federated learning models.

- **DNA+MRI Multimodal Dataset:**
  - **Notebook:** `src/Standard_FedQNN_DNA+MRI.ipynb`
  - **Description:** It is used as a MoE with Multimodaility leveraging both DNA Sequence and MRI scans data to develop and evaluate models for detecting and interpreting tumors and dna classes.
  
## Datasets

Download the datasets using the following commands:

```bash
# DNA Sequence Dataset
kaggle datasets download -d nageshsingh/dna-sequence-dataset
mkdir -p data/DNA
unzip dna-sequence-dataset.zip -d data/DNA
rm dna-sequence-dataset.zip

# MRI Scan Dataset
kaggle datasets download -d masoudnickparvar/brain-tumor-mri-dataset
mkdir -p data/MRI
unzip brain-tumor-mri-dataset.zip -d data/MRI
rm brain-tumor-mri-dataset.zip
```  
